import surfex
from surfex.util import warning
import copy
import numpy as np
from datetime import timedelta


class Variable(object):

    """
    New combined variable
    """

    def __init__(self, var_type, var_dict, initial_basetime, debug=False, prefer_forecast=True):

        self.var_type = var_type
        if self.var_type == "netcdf":
            mandatory = ["name", "fcint", "offset", "filepattern"]
        elif self.var_type == "grib1":
            mandatory = ["parameter", "type", "level", "tri", "fcint", "offset", "filepattern"]
            pass
        elif self.var_type == "grib2":
            mandatory = ["discipline", "parameterCategory", "parameterNumber", "levelType", "level",
                         "typeOfStatisticalProcessing", "fcint", "offset", "filepattern"]
        elif self.var_type == "surfex":
            mandatory = ["varname", "fcint", "offset",  "filepattern"]
            # , "patches", "layers", "accumulated", "fcint", "offset", "file_inc", "filepattern",
            #         "fileformat", "filetype"]
        elif self.var_type == "fa":
            mandatory = ["name", "fcint", "offset", "filepattern"]
        elif self.var_type == "obs":
            mandatory = ["filetype", "fcint", "offset"]
        else:
            raise NotImplementedError

        for i in range(0, len(mandatory)):
            if mandatory[i] not in var_dict:
                raise Exception(var_type + " variable must have attribute " + mandatory[i] +
                                " var_dict:" + str(var_dict))

        self.var_dict = copy.deepcopy(var_dict)
        interval = 3600
        if "timestep" in var_dict:
            interval = var_dict["timestep"]
        self.interval = interval
        if "filepattern" in var_dict:
            self.filepattern = var_dict["filepattern"]
        else:
            raise Exception("No filepattern provided")
        self.initial_basetime = initial_basetime
        self.fcint = int(self.var_dict["fcint"])
        self.offset = int(self.var_dict["offset"])
        self.prefer_forecast = prefer_forecast
        if "prefer_forecast" in self.var_dict:
            self.prefer_forecast = self.var_dict["prefer_forecast"]

        if self.offset > self.fcint:
            raise Exception("You can not have larger offset than the frequency of forecasts " +
                            str(self.offset) + " > " + str(self.fcint))

        self.debug = debug
        if self.debug:
            surfex.debug(__file__, self.__class__.__name__, "Constructed " + self.__class__.__name__ + " for " +
                         str(self.var_dict))

    def get_filename(self, validtime,  previoustime=None):
        if self.debug:
            surfex.debug(__file__, self.__class__.get_filename.__name__, "Set basename for filename")
        basetime = self.get_basetime(validtime,  previoustime=previoustime)
        if previoustime is not None:
            validtime = previoustime
        return surfex.file.parse_filepattern(self.filepattern, basetime, validtime, debug=self.debug)

    def get_filehandler(self, validtime, cache=None, previoustime=None):

        filename = self.get_filename(validtime, previoustime=previoustime)
        if cache is not None and cache.file_open(filename):
            file_handler = cache.get_file_handler(filename)
        else:
            if self.var_type == "netcdf":
                file_handler = surfex.netcdf.Netcdf(filename, debug=self.debug)
            elif self.var_type == "grib1" or self.var_type == "grib2":
                file_handler = surfex.grib.Grib(filename, debug=self.debug)
            elif self.var_type == "surfex":
                fileformat = None
                if "fileformat" in self.var_dict:
                    fileformat = self.var_dict["fileformat"]
                filetype = None
                if "filetype" in self.var_dict:
                    filetype = self.var_dict["filetype"]
                geo_in = None
                if "geo_input" in self.var_dict:
                    geo_in = self.var_dict["geo_input"]
                file_handler = surfex.file.get_surfex_io_object(filename, fileformat=fileformat,
                                                                filetype=filetype, geo=geo_in, debug=self.debug)
            elif self.var_type == "obs":
                var_dict = self.var_dict
                var_dict = {"set": var_dict}
                basetime = self.get_basetime(validtime)
                file_handler = surfex.get_datasources(basetime, var_dict)[0]
            else:
                raise NotImplementedError

            if cache is not None:
                cache.set_file_handler(filename, file_handler)

        if self.debug:
            surfex.debug(__file__, self.__class__.get_filehandler.__name__, "var_type:", self.var_type)
            surfex.debug(__file__, self.__class__.get_filehandler.__name__, "filename:", filename)
        return file_handler, filename

    def read_var_points(self, var, geo, validtime, previoustime=None, cache=None):

        interpolation = "bilinear"
        if self.var_type != "obs":
            if "interpolator" in self.var_dict:
                interpolation = self.var_dict["interpolator"]

        if self.debug:
            surfex.debug(__file__, self.__class__.read_variable.__name__, "set basetime from ", validtime)

        # TODO put this in a netcdf variable and we don't need this any more
        kwargs = {}
        if self.var_type == "netcdf":
            kwargs.update({"level": None})
            kwargs.update({"units": None})
            if "level" in self.var_dict:
                kwargs.update({"level": [self.var_dict["level"]]})
            if "units" in self.var_dict:
                kwargs.update({"units": str([self.var_dict["units"]][0])})

        filehandler, filename = self.get_filehandler(validtime, cache=cache, previoustime=previoustime)
        if previoustime is not None:
            validtime = previoustime
            if self.debug:
                surfex.debug(__file__, self.__class__.read_variable.__name__, "new validtime to read previous time",
                             validtime)

        if filehandler is None:
            surfex.warning("No file handler exist for this time step")
            field = np.array([len(geo.lons)])
            field = field.fill(np.nan)
        else:
            id_str = None
            if cache is not None:
                id_str = cache.generate_id(self.var_type, var, filename, validtime)

            if cache is not None and cache.is_saved(id_str):
                field = cache.saved_fields[id_str]
                surfex.info("Using cached value for " + id_str)
            else:
                if self.var_type == "obs":
                    times, field, stids = filehandler.points(geo, validtime=validtime)
                else:
                    field, interpolator = filehandler.points(var, geo, interpolation=interpolation,
                                                             validtime=validtime)

                    field = self.rotate_geographic_wind(field, interpolator)
                if self.debug and field is not None:
                    surfex.debug(__file__, self.__class__.read_variable.__name__, "field.shape", field.shape)

                if cache is not None:
                    if self.debug:
                        surfex.debug(__file__, self.__class__.read_variable.__name__, "ID_STRING", id_str)
                    cache.save_field(id_str, field)
        return field

    def set_var(self, validtime=None):
        accumulated = False
        if self.var_type == "netcdf":
            name = self.var_dict["name"]
            if "accumulated" in self.var_dict:
                accumulated = self.var_dict["accumulated"]
            level = None
            if "level" in self.var_dict:
                level = self.var_dict["level"]
                if not isinstance(level, list):
                    level = [level]
            units = None
            if "units" in self.var_dict:
                units = str([self.var_dict["units"]][0])
            member = None
            if "member" in self.var_dict:
                member = self.var_dict["member"]
                if member is not None:
                    if not isinstance(member, list):
                        member = [member]
            var = surfex.NetCDFReadVariable(name, level=level, units=units, member=member)
        elif self.var_type == "grib1":
            par = self.var_dict["parameter"]
            typ = self.var_dict["type"]
            level = self.var_dict["level"]
            tri = self.var_dict["tri"]
            if tri == 4:
                accumulated = True
            var = surfex.grib.Grib1Variable(par, typ, level, tri, debug=self.debug)
        elif self.var_type == "grib2":
            discipline = self.var_dict["discipline"]
            pc = self.var_dict["parameterCategory"]
            pn = self.var_dict["parameterNumber"]
            lt = self.var_dict["levelType"]
            lev = self.var_dict["level"]
            tsp = -1
            if "typeOfStatisticalProcessing" in self.var_dict:
                tsp = self.var_dict["typeOfStatisticalProcessing"]
            if tsp == 1:
                accumulated = True
            var = surfex.grib.Grib2Variable(discipline, pc, pn, lt, lev, tsp=tsp, debug=self.debug)
        elif self.var_type == "surfex":
            varname = self.var_dict["varname"]
            layers = None
            if "layers" in self.var_dict:
                layers = self.var_dict["layers"]
            patches = None
            if "patches" in self.var_dict:
                patches = self.var_dict["patches"]
            if "accumulated" in self.var_dict:
                accumulated = self.var_dict["accumulated"]
            datatype = None
            if "datatype" in self.var_dict:
                datatype = self.var_dict["datatype"]

            basetime = self.get_basetime(validtime)
            var = surfex.file.SurfexFileVariable(varname, validtime=validtime, patches=patches, layers=layers,
                                                 basetime=basetime,
                                                 interval=self.interval, datatype=datatype)
        elif self.var_type == "fa":
            var = self.var_dict["name"]
        elif self.var_type == "obs":
            if "varname" in self.var_dict:
                var = self.var_dict["varname"]
            else:
                var = None
        else:
            raise NotImplementedError

        instant = 0
        if accumulated:
            instant = self.interval
        if "instant" in self.var_dict:
            instant = self.var_dict["instant"]
        if "prefer_forecast" in self.var_dict:
            self.prefer_forecast = self.var_dict["prefer_forecast"]

        return accumulated, instant, var

    def read_variable(self, geo, validtime, cache):

        # Set variable info
        accumulated, instant, var = self.set_var(validtime=validtime)
        previous_field = None
        if accumulated:
            # Re-read field
            previoustime = validtime - timedelta(seconds=self.interval)
            # Don't read if previous time is older than the very first basetime
            if previoustime >= self.initial_basetime:
                if self.debug:
                    surfex.debug(__file__, self.__class__.read_variable.__name__, "Re-read ", previoustime)
                previous_field = self.read_var_points(var, geo, validtime=validtime, previoustime=previoustime,
                                                      cache=cache)
            else:
                previous_field = np.zeros([geo.npoints])
        elif instant > 0:
            previous_field = np.zeros([geo.npoints])

        field = self.read_var_points(var, geo, validtime=validtime, cache=cache)

        # Deaccumulate if either two files are read or if instant is > 0.
        if accumulated or instant > 0:
            field = self.deaccumulate(field, previous_field, instant)
            if any(field[field < 0.]):
                raise Exception("Negative accumulated value found for " + var)
        return field

    def print_variable_info(self):
        surfex.debug(__file__, self.__class__.print_variable_info.__name__, ":" + str(self.var_dict) + ":")

    def deaccumulate(self, field, previous_field, instant):

        if self.debug:
            surfex.debug(__file__, self.__class__.deaccumulate.__name__, "Deaccumulate field with: ", instant)
        if field is None:
            surfex.warning("Field is not read properly")
            return None
        else:
            field = np.subtract(field, previous_field)
            if any(field[field < 0.]):
                neg = []
                for i in range(0, field.shape[0]):
                    if field[i] < 0.:
                        neg.append(field[i])
                neg = np.asarray(neg)
                warning("Deaccumulated field has " + str(neg.shape[0]) + " negative lowest:"
                        + str(np.nanmin(neg)) + " mean: " + str(np.nanmean(neg)))
            field[field < 0.] = 0
            if any(field[field < 0.]):
                raise Exception("Should not be negative values")
            if float(instant) != 0.:
                field = np.divide(field, float(instant))

            return field

    def get_basetime(self, validtime, previoustime=None, allow_different_basetime=False):

        if self.offset < 0:
            raise Exception("Negative offset does not make sense here")

        # Take offset into account
        first = False
        offset = self.offset

        basetime = validtime - timedelta(seconds=offset)
        if basetime <= self.initial_basetime:
            first = True
            basetime = self.initial_basetime

        if self.debug:
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "             validtime: ", validtime)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "          previoustime: ", previoustime)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "                 first: ", first)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "                 fcint: ", self.fcint)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "                offset: ", self.offset)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "      initial_basetime: ",
                         self.initial_basetime)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "  Basetime with offset: ", basetime)

        # Modify based on fcint
        seconds_since_midnight = \
            int((basetime - basetime.replace(hour=0, minute=0, second=0, microsecond=0)).total_seconds())
        if seconds_since_midnight == 86400:
            seconds_since_midnight = 0
        basetime_inc = int(seconds_since_midnight / int(timedelta(seconds=self.fcint).seconds))

        prefer_forecast = timedelta(seconds=0)
        if seconds_since_midnight == basetime_inc * int(timedelta(seconds=self.fcint).seconds):
            if first:
                if self.debug:
                    surfex.debug(__file__, self.__class__.get_basetime.__name__, "First basetime")
            else:
                if self.prefer_forecast:
                    if self.debug:
                        surfex.debug(__file__, self.__class__.get_basetime.__name__,
                                     "Prefer forecasts instead of analyis")
                    prefer_forecast = timedelta(seconds=self.fcint)
                else:
                    if self.debug:
                        surfex.debug(__file__, self.__class__.get_basetime.__name__,
                                     "Prefer analysis instead of forecast")

        fcint = timedelta(seconds=self.fcint)
        basetime = basetime.replace(hour=0, minute=0, second=0, microsecond=0) + \
                                   (basetime_inc * fcint) - prefer_forecast

        if previoustime is not None:
            if allow_different_basetime:
                raise NotImplementedError

        if self.debug:
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "seconds_since_midnight: ",
                         seconds_since_midnight)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "         cycle seconds:",
                         basetime_inc * int(timedelta(seconds=self.fcint).seconds))
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "       prefer_forecast: ",
                         self.prefer_forecast)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "   prefer_forecast_inc: ",
                         prefer_forecast)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "          basetime_inc:", basetime_inc)
            surfex.debug(__file__, self.__class__.get_basetime.__name__, "           Basetime is:", basetime)
        return basetime

    def rotate_geographic_wind(self, field, interpolator):
        rotate_wind = False
        if "rotate_to_geographic" in self.var_dict:
            rotate_wind = self.var_dict["rotate_to_geographic"]
        if rotate_wind:
            field = interpolator.rotate_wind_to_geographic(field)
            return field
        else:
            return field



__version__ = '0.0.1a19'
import sys
import os
if "PYSURFEX_EXPERIMENT" in os.environ:
    exp_path = os.environ["PYSURFEX_EXPERIMENT"]
    sys.path.insert(0, exp_path)
else:
    exp_path = None

def use_local_file(exp_path, f):
   if exp_path is not None:
       if os.path.exists(exp_path + "/surfex_exp/" + f):
            return False
   return True


if use_local_file(exp_path, "file.py"):
    from .file import *
if use_local_file(exp_path, "geo.py"):
    from .geo import *
if use_local_file(exp_path, "read.py"):
    from .read import *
if use_local_file(exp_path, "run.py"):
    from .run import *
if use_local_file(exp_path, "assim.py"):
    from .assim import *
if use_local_file(exp_path, "obs.py"):
    from .obs import *
if use_local_file(exp_path, "forcing.py"):
    from .forcing import *
if use_local_file(exp_path, "netcdf.py"):
    from .netcdf import *
if use_local_file(exp_path, "grib.py"):
    from .grib import *
if use_local_file(exp_path, "bufr.py"):
    from .bufr import *
if use_local_file(exp_path, "variable.py"):
    from .variable import *
if use_local_file(exp_path, "util.py"):
    from .util import *
if use_local_file(exp_path, "cache.py"):
    from .cache import *
if use_local_file(exp_path, "interpolation.py"):
    from .interpolation import *
if use_local_file(exp_path, "titan.py"):
    from .titan import *
if use_local_file(exp_path, "timeseries.py"):
    from .timeseries import *
if use_local_file(exp_path, "obsmon.py"):
    from .obsmon import *
if use_local_file(exp_path, "fa.py"):
    from .fa import *
if use_local_file(exp_path, "cli.py"):
    from .cli import *
if use_local_file(exp_path, "namelist.py"):
    from .namelist import *
if use_local_file(exp_path, "configuration.py"):
    from .configuration import *

if exp_path is not None:
    print("Overriding from " + exp_path)
    from surfex_exp import *



import sys
import os
from datetime import datetime
import collections
import toml


def error(message):
    """ Write error message to console and abort """
    print("\033[1;31mError: " + message + "\033[0m")
    sys.exit(1)


def debug(path, name, *args):
    """ Write debug to console """
    message = ""
    for arg in args:
        message = message + " " + str(arg)
    print("\033[1;34m" + str(path) + "::" + name + " " + message + "\033[0m")


def info(message, level=0):
    """ Write a information message to console """
    if level < 1:
        print("\033[1;92mINFO: " + message + "\033[0m")


def warning(message):
    """ Write a warning message to console """
    print("\033[1;33mWarning: " + message + "\033[0m")


def unixtime_to_datenum(time):

    """ Converts unixtime into datenum

    Arguments:
      time (int): unixtime in seconds since 1970

    Returns:
      int: datenum value
    """
   
    dt = datetime.utcfromtimestamp(time)
    try:
        from matplotlib import dates
        dt2 = dates.date2num(dt)
        return dt2
    except ImportError:
        raise Exception("You need to have dates installed")


class YamlReaderError(Exception):
    pass


def data_merge(a, b):
    """merges b into a and return merged result

    NOTE: tuples and arbitrary objects are not handled as it is totally ambiguous what should happen"""
    key = None
    # ## debug output
    # sys.stderr.write("DEBUG: %s to %s\n" %(b,a))
    try:
        if a is None or isinstance(a, str) or isinstance(a, int) or isinstance(a, float):
            # border case for first run or if a is a primitive
            a = b
        elif isinstance(a, list):
            # lists can be only appended
            if isinstance(b, list):
                # merge lists
                a.extend(b)
            else:
                # append to list
                a.append(b)
        elif isinstance(a, dict):
            # dicts must be merged
            if isinstance(b, dict):
                for key in b:
                    if key in a:
                        a[key] = data_merge(a[key], b[key])
                    else:
                        a[key] = b[key]
            else:
                raise YamlReaderError('Cannot merge non-dict "%s" into dict "%s"' % (b, a))
        else:
            raise YamlReaderError('NOT IMPLEMENTED "%s" into "%s"' % (b, a))
    except TypeError as e:
        raise YamlReaderError('TypeError "%s" in key "%s" when merging "%s" into "%s"' % (e, key, b, a))
    return a


def merge_toml_env(old_env, mods):
    # print(mods)
    return deep_update(old_env, mods)


def merge_toml_env_from_files(toml_files):
    merged_env = {}
    for toml_file in toml_files:
        if os.path.exists(toml_file):
            # print(toml_file)
            modification = toml.load(open(toml_file, "r"))
            # print(modification)
            merged_env = merge_toml_env(merged_env, modification)
            # print(merged_env)
        else:
            print("WARNING: File not found " + toml_file)
    return merged_env


def deep_update(source, overrides):
    """
    Update a nested dictionary or similar mapping.
    Modify ``source`` in place.
    """
    for key, value in overrides.items():
        if isinstance(value, collections.Mapping) and value:
            returned = deep_update(source.get(key, {}), value)
            # print("Returned:", key, returned)
            source[key] = returned
        else:
            override = overrides[key]
            # print("Override:", key, override)

            source[key] = override

    return source



import unittest
from datetime import datetime, timedelta
import yaml

from surfex.variable import Variable


class TestVariable(unittest.TestCase):
   
    def setUp(self):
        with open("test/fixtures/config.yml", 'r') as cfgf:
            self.cfg = yaml.safe_load(cfgf)

    def test_open_new_file_nc(self):
        debug = True
        initialtime = datetime(2019, 11, 13, 0)
        intervall = 3600
        case = "netcdf"

        var_dict = self.cfg[case]
        var_type = case
        for i in range(11):
            with self.subTest(i=i):
                validtime = initialtime + timedelta(seconds=intervall*i)
                previoustime = validtime - timedelta(seconds=intervall)
                variable = Variable(var_type, var_dict, initialtime, debug=debug)
                previous_filename = variable.get_filename(validtime, previoustime=previoustime)
                filename = variable.get_filename(validtime)
                self.assertEqual(filename, var_dict['blueprint'][i])
                if i > 0:
                    self.assertEqual(previous_filename, var_dict['blueprint_previous'][i])

    def test_open_new_file_grib1(self):
        debug = True
        initialtime = datetime(2019, 11, 13, 0)
        intervall = 3600
        case = "grib1"

        var_dict = self.cfg[case]
        var_type = case
        for i in range(11):
            with self.subTest(i=i):
                validtime = initialtime + timedelta(seconds=intervall*i)
                previoustime = validtime - timedelta(seconds=intervall)
                variable = Variable(var_type, var_dict, initialtime, debug=debug)
                previous_filename = variable.get_filename(validtime, previoustime=previoustime)
                filename = variable.get_filename(validtime)
                self.assertEqual(filename, var_dict['blueprint'][i])
                if i > 0:
                    self.assertEqual(previous_filename, var_dict['blueprint_previous'][i])

    def test_open_new_file_grib2(self):
        debug = True
        initialtime = datetime(2019, 11, 13, 2)
        intervall = 3600
        case = "grib2"

        var_dict = self.cfg[case]
        var_type = case
        for i in range(11):
            with self.subTest(i=i):
                validtime = initialtime + timedelta(seconds=intervall*i)
                previoustime = validtime - timedelta(seconds=intervall)
                variable = Variable(var_type, var_dict, initialtime, debug=debug)
                previous_filename = variable.get_filename(validtime, previoustime=previoustime)
                filename = variable.get_filename(validtime)
                self.assertEqual(filename, var_dict['blueprint'][i])
                if i > 0:
                    self.assertEqual(previous_filename, var_dict['blueprint_previous'][i])

    def test_open_new_file_an(self):
        debug = True
        initialtime = datetime(2019, 11, 13, 0)
        intervall = 3600
        case = "met_nordic"

        var_dict = self.cfg[case]
        var_type = case
        if var_type == "met_nordic":
            var_type = "netcdf"
        for i in range(11):
            with self.subTest(i=i):
                validtime = initialtime + timedelta(seconds=intervall*i)
                variable = Variable(var_type, var_dict, initialtime, debug=debug)
                filename = variable.get_filename(validtime)
                self.assertEqual(filename, var_dict['blueprint'][i])

    def test_open_new_file_fail(self):
        debug = True
        initialtime = datetime(2019, 11, 13, 0)
        case = "met_nordic"
        var_dict = self.cfg[case]
        var_dict["offset"] = 7200
        var_type = case
        if var_type == "met_nordic":
            var_type = "netcdf"
        with self.assertRaises(Exception):
            Variable(var_type, var_dict, initialtime, debug=debug)


if __name__ == "__main__":
    unittest.main()



import surfex
import numpy as np
import gridpp


class Interpolation(object):

    def __init__(self, operator, geo_in, geo_out, debug=False):

        self.debug = debug
        self.operator = operator
        self.geo_in = geo_in
        self.geo_out = geo_out
        if self.geo_out is None:
            raise Exception("You can not interpolate without specifying an output geometry")

        # Input
        if self.geo_in is not None:
            grid_lons = np.array(geo_in.lons)
            grid_lats = np.array(geo_in.lats)
            self.var_lons = grid_lons
            self.var_lats = grid_lats
            self.identical = self.geo_out.is_identical(self.geo_in)
            if self.debug:
                surfex.debug(__file__, self.__class__.__name__, "grid_lons.shape", grid_lons.shape)
                surfex.debug(__file__, self.__class__.__name__, "grid_lats.shape", grid_lats.shape)
            self.grid = gridpp.Grid(grid_lons, grid_lats)
        else:
            self.var_lons = None
            self.var_lats = None
            self.grid = None
            self.identical = False

        # Output
        lons = np.array(self.geo_out.lonlist)
        lats = np.array(self.geo_out.latlist)
        self.npoints = self.geo_out.npoints
        if self.debug:
            surfex.debug(__file__, self.__class__.__name__, "Output lons shape:", lons.shape)
            surfex.debug(__file__, self.__class__.__name__, "Output lats shape:", lats.shape)
        self.points = gridpp.Points(lons, lats)

    def interpolate(self, field2d, undefined=None):

        if field2d is None and undefined is not None:
            return np.full((self.geo_out.nlons * self.geo_out.nlats), undefined)
        elif field2d is None:
            raise Exception("You try to interpolate a missing field!")
        else:
            if self.debug:
                surfex.debug(__file__, self.__class__.interpolate.__name__, "field2d.shape", field2d.shape)
                surfex.debug(__file__, self.__class__.interpolate.__name__, "gridpp.__file__", gridpp.__file__)
            if self.identical or self.operator == "none":
                if self.operator == "none":
                    if not self.identical:
                        raise Exception("Input domain and ouput domain differ. You must interpolate!")
                    surfex.info("No interpolation chosen")
                else:
                    surfex.info("Input and output domain are identical. No interpolation is needed")
                interpolated_field = field2d.reshape(self.npoints)
            else:
                sub_lons, sub_lats = self.geo_out.subset(self.geo_in)
                if len(sub_lons) == 0 and len(sub_lats) == 0:
                    surfex.info("Doing \"" + self.operator + "\" interpolation for " + str(self.npoints) + " points")
                    if self.operator == "nearest":
                        interpolated_field = gridpp.nearest(self.grid, self.points, field2d)
                    elif self.operator == "bilinear":
                        interpolated_field = gridpp.bilinear(self.grid, self.points, field2d)
                    elif self.operator == "none":
                        interpolated_field = field2d.reshape(self.npoints)
                    else:
                        raise NotImplementedError(self.operator)
                else:
                    surfex.info("Output domain is a subset of input domain")
                    new_field = np.ndarray([len(sub_lons), len(sub_lats)])
                    for i in range(0, len(sub_lons)):
                        for j in range(0, len(sub_lats)):
                            new_field[i, j] = field2d[sub_lons[i], sub_lats[j]]
                    interpolated_field = new_field.reshape(self.npoints)
            return interpolated_field

    def rotate_wind_to_geographic(self):
        pass

    @staticmethod
    def distance(lon1, lat1, lon2, lat2):
        """
        Computes the great circle distance between two points using the
        haversine formula. Values can be vectors.
        """
        # Convert from degrees to radians
        pi = 3.14159265
        lon1 = lon1 * 2 * pi / 360.
        lat1 = lat1 * 2 * pi / 360.
        lon2 = lon2 * 2 * pi / 360.
        lat2 = lat2 * 2 * pi / 360.
        dlon = lon2 - lon1
        dlat = lat2 - lat1
        a = np.sin(dlat / 2.) ** 2 + np.cos(lat1) * np.cos(lat2) * np.sin(dlon / 2.) ** 2
        c = 2 * np.arcsin(np.sqrt(a)) * 6.367e6
        return c

    def alpha_grid_rot(self):
        lon = self.var_lons
        lat = self.var_lats
        nx = lat.shape[0]
        # ny = lat.shape[1]
        dlon = np.zeros(lat.shape)
        dlat = np.zeros(lat.shape)
        i1 = np.arange(nx - 1)

        dlon[0:-1, :] = np.sign(lon[i1 + 1, :] - lon[i1, :]) * self.distance(lon[i1, :], lat[i1, :], lon[i1 + 1, :],
                                                                             lat[i1, :])
        dlat[0:-1, :] = -np.sign(lat[i1 + 1, :] - lat[i1, :]) * self.distance(lon[i1, :], lat[i1, :], lon[i1, :],
                                                                              lat[i1 + 1, :])

        dlon[-1, :] = np.sign(lon[-1, :] - lon[-2, :]) * self.distance(lon[-2, :], lat[-2, :], lon[-1, :], lat[-2, :])
        dlat[-1, :] = -np.sign(lat[-1, :] - lat[-2, :]) * self.distance(lon[-2, :], lat[-2, :], lon[-2, :], lat[-1, :])

        alpha = np.rad2deg(np.arctan2(dlon, dlat))
        return alpha
